/* charcell.c - handle character cell data
 *
 * Copyright (C) 2005  Jochen Voss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: charcell.c 6725 2005-11-09 23:15:58Z voss $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include "base.h"
#include "backend.h"


struct combination {
  struct charcell  cc1;
  Uint16  c2;
  struct charcell  res;
};

static  struct combination *dict = NULL;
static  unsigned  dict_allocated = 0;
static  unsigned  dict_used = 0;

static  Uint16  new_code = 0;


#include "uc_combining.h"

static int
charcell_equal (struct charcell cc1, struct charcell cc2)
{
  return  cc1.c == cc2.c && cc1.attr == cc2.attr && cc1.colors == cc2.colors;
}

struct charcell
charcell_combine (struct charcell cc1, Uint16 c2)
{
  unsigned  i;

  for (i=0; i<dict_used; ++i) {
    if (charcell_equal (dict[i].cc1, cc1) && dict[i].c2 == c2) {
      return  dict[i].res;
    }
  }
  if (dict_used >= dict_allocated) {
    dict_allocated += 16;
    dict = xrealloc (dict, dict_allocated*sizeof(struct combination));
  }
  dict[dict_used].cc1 = cc1;
  dict[dict_used].c2 = c2;
  dict[dict_used].res.c = new_code++;
  dict[dict_used].res.attr = cc1.attr | JV_ATTR_COMBINED;
  dict[dict_used].res.colors = cc1.colors;
  return  dict[dict_used++].res;
}

int
charcell_disband (struct charcell cc, Uint16 *c1, Uint16 *c2)
{
  unsigned  i;

  for (i=0; i<dict_used; ++i) {
    if (charcell_equal (dict[i].res, cc)) {
      *c1 = dict[i].cc1.c;
      *c2 = dict[i].c2;
      return  1;
    }
  }

  return  0;
}

char *
charcell_attr_name (Uint8 attr)
/* Return a string representation of the attribute set 'attr'.
 * The caller must free the returned string with 'xfree'.  */
{
  const char *attrs[8] = {
    "blink", "bright", "dim", "reverse",
    "underline", "acs", "combined", "cursor"
  };
  char  buffer [256];
  int  i, mask, first;

  buffer[0] = '\0';
  first = 1;
  for (i=0,mask=1; i<8; ++i,mask<<=1) {
    if (! (mask&attr))  continue;
    if (first) {
      first = 0;
    } else {
      strcat (buffer, ", ");
    }
    strcat (buffer, attrs[i]);
  }
  if (! buffer[0]) {
    strcat (buffer, "no attributes");
  }
  return  xstrdup(buffer);
}
