/* color.c - jvterm color handling
 *
 * Copyright (C) 2005  Jochen Voss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: color.c 6725 2005-11-09 23:15:58Z voss $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <SDL.h>

#include "base.h"
#include "backend.h"


/* tables with RGB values for the colors */
static SDL_Color colors_normal[8] = {
  { 0x00, 0x00, 0x00, 0 },	/* black */
  { 0xB0, 0x00, 0x00, 0 },	/* red */
  { 0x00, 0xB0, 0x00, 0 },	/* green */
  { 0xB0, 0xB0, 0x00, 0 },	/* yellow */
  { 0x00, 0x00, 0xB0, 0 },	/* blue */
  { 0xB0, 0x00, 0xB0, 0 },	/* magenta */
  { 0x00, 0xB0, 0xB0, 0 },	/* cyan */
  { 0xB0, 0xB0, 0xB0, 0 }	/* white */
};
static SDL_Color colors_dim[8] = {
  { 0x00, 0x00, 0x00, 0 },	/* dim black */
  { 0x60, 0x00, 0x00, 0 },	/* dim red */
  { 0x00, 0x60, 0x00, 0 },	/* dim green */
  { 0x60, 0x60, 0x00, 0 },	/* dim yellow */
  { 0x00, 0x00, 0x70, 0 },	/* dim blue */
  { 0x60, 0x00, 0x60, 0 },	/* dim magenta */
  { 0x00, 0x60, 0x60, 0 },	/* dim cyan */
  { 0x60, 0x60, 0x60, 0 }	/* dim white */
};
static SDL_Color colors_bright[8] = {
  { 0x00, 0x00, 0x00, 0 },	/* bright black */
  { 0xFF, 0x00, 0x00, 0 },	/* bright red */
  { 0x00, 0xFF, 0x00, 0 },	/* bright green */
  { 0xFF, 0xFF, 0x00, 0 },	/* bright yellow */
  { 0x10, 0x10, 0xFF, 0 },	/* bright blue */
  { 0xFF, 0x00, 0xFF, 0 },	/* bright magenta */
  { 0x00, 0xFF, 0xFF, 0 },	/* bright cyan */
  { 0xFF, 0xFF, 0xFF, 0 }	/* bright white */
};
static SDL_Color colors_extra[1] = {
  { 0x20, 0x20, 0x20, 0 }	/* dark gray */
};


const char *
color_name (Uint8 col)
/* Return the name of color 'col' as a string.  */
{
  const char *colors[8] = {
    "black", "red", "green", "yellow", "blue", "magenta", "cyan", "white"
  };
  return  col>7 ? "unknown" : colors[col];
}

void
color_decode (struct charcell cc, SDL_Color **fg, SDL_Color **bg)
{
  int  fg_index, bg_index;
  SDL_Color *forecol, *backcol;
  
  if (cc.attr & JV_ATTR_REVERSE) {
    fg_index = GET_COLOR_BG(cc.colors);
    bg_index = GET_COLOR_FG(cc.colors);
  } else {
    fg_index = GET_COLOR_FG(cc.colors);
    bg_index = GET_COLOR_BG(cc.colors);
  }
    
  if (cc.attr&JV_ATTR_BRIGHT) {
    if (fg_index == JV_COLOR_BLACK && bg_index == JV_COLOR_BLACK) {
      forecol = colors_extra + 0;
    } else {
      forecol = colors_bright + fg_index;
    }
    if (bg_index == JV_COLOR_WHITE
	|| bg_index == fg_index) {
      backcol = colors_normal + bg_index;
    } else {
      backcol = colors_bright + bg_index;
    }
  } else if (cc.attr&JV_ATTR_DIM) {
    if (bg_index == fg_index) {
      if (fg_index == JV_COLOR_BLACK) {
	forecol = colors_extra + 0;
      } else {
	forecol = colors_normal + fg_index;
      }
    } else {
      forecol = colors_dim + fg_index;
    }
    backcol = colors_dim + bg_index;
  } else {
    if (fg_index == JV_COLOR_BLACK && bg_index == JV_COLOR_BLACK) {
      forecol = colors_extra + 0;
    } else {
      forecol = colors_normal + fg_index;
    }
    if (bg_index == fg_index) {
      backcol = colors_dim + bg_index;
    } else {
      backcol = colors_normal + bg_index;
    }
  }
  *fg = forecol;
  *bg = backcol;
}
