/* image.c - jvterm image handling
 *
 * Copyright (C) 2004  Jochen Voss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: image.c 6725 2005-11-09 23:15:58Z voss $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <SDL_image.h>
#include <SDL_rotozoom.h>

#include "base.h"
#include "backend.h"


struct image *
load_image (const char *file_name)
{
  struct image *im;

  im = xnew (struct image);
  
  im->orig = IMG_Load (file_name);
  if (! im->orig) {
    /* TODO: convert filename to UTF-8 and enable non-ASCII file names */
    error ("cannot load image file: %s", SDL_GetError ());
    return  NULL;
  } else {
    /* TODO: convert filename to UTF-8 and enable non-ASCII file names */
    message ("loading image file \"%s\"", file_name);
  }
  im->scaled = NULL;
  
  if (im->orig->w <= screen_width()) {
    im->xoffs = 0;
  } else {
    im->xoffs = 0.5*(im->orig->w - screen_width());
  }
  if (im->orig->h <= screen_height()) {
    im->yoffs = 0;
  } else {
    im->yoffs = 0.5*(im->orig->h - screen_height());
  }

  return  im;
}

void
delete_image (struct image *im)
{
  if (im->scaled)  SDL_FreeSurface (im->scaled);
  SDL_FreeSurface (im->orig);
  xfree (im);
}

SDL_Surface *
image_get (struct image *im)
{
  if (im->scaled)  return  im->scaled;
  return  im->orig;
}

void
image_scale (struct image *im, double scale)
{
  double  qx, qy;

  if (image_get(im)->w <= screen_width()) {
    qx = 0.5;
  } else {
    qx = im->xoffs / (double)(image_get(im)->w-screen_width());
  }
  if (image_get(im)->h <= screen_height()) {
    qy = 0.5;
  } else {
    qy = im->yoffs / (double)(image_get(im)->h-screen_height());
  }

  if (im->scaled)  SDL_FreeSurface (im->scaled);
  im->scaled = zoomSurface (im->orig, scale, scale, 1);
  
  if (im->scaled->w <= screen_width()) {
    im->xoffs = 0;
  } else {
    im->xoffs = qx*(im->scaled->w - screen_width());
  }
  if (im->scaled->h <= screen_height()) {
    im->yoffs = 0;
  } else {
    im->yoffs = qy*(im->scaled->h - screen_height());
  }
}

void
image_scroll (struct image *im, int dx, int dy)
{
  im->xoffs += dx;
  if (im->xoffs < 0) {
    im->xoffs = 0;
  } else if (im->xoffs + screen_width() > image_get(im)->w) {
    im->xoffs = image_get(im)->w - screen_width();
  }

  im->yoffs += dy;
  if (im->yoffs < 0) {
    im->yoffs = 0;
  } else if (im->yoffs + screen_height() > image_get(im)->h) {
    im->yoffs = image_get(im)->h - screen_height();
  }
}
