/* keyboard.c - jvterm client to debug keyboard problems
 *
 * Copyright (C) 2005  Jochen Voss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id$
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include "base.h"
#include "backend.h"
#include "ui.h"


struct client *keyclient;

struct keyboard_client_data {
  struct buffer *buf;
};

static void *
keyboard_init (struct buffer *buf, void *init_data)
{
  struct keyboard_client_data *dcd;

  dcd = xnew (struct keyboard_client_data);
  dcd->buf = buf;
  
  return  dcd;
}

static void
keyboard_exit (void *data)
{
  struct keyboard_client_data *dcd = data;
  xfree (dcd);
}

static void
keyboard_activate (void *data)
{
  struct keyboard_client_data *dcd = data;
  buffer_connect (dcd->buf);
  buffer_puts (dcd->buf, "keycode debugger, press q to quit\n",
	       0, JV_COLOR_GREEN);
}

static void
cat_modifier (SDLMod mod, char *buffer)
{
  if (mod & KMOD_LSHIFT)  strcat (buffer, "LSHIFT-");
  if (mod & KMOD_RSHIFT)  strcat (buffer, "RSHIFT-");
  if (mod & KMOD_LCTRL)  strcat (buffer, "LCTRL-");
  if (mod & KMOD_RCTRL)  strcat (buffer, "RCTRL-");
  if (mod & KMOD_LALT)  strcat (buffer, "LALT-");
  if (mod & KMOD_RALT)  strcat (buffer, "RALT-");
  if (mod & KMOD_LMETA)  strcat (buffer, "LMETA-");
  if (mod & KMOD_RMETA)  strcat (buffer, "RMETA-");
  if (mod & KMOD_NUM)  strcat (buffer, "NUM-");
  if (mod & KMOD_CAPS)  strcat (buffer, "CAPS-");
  if (mod & KMOD_MODE)  strcat (buffer, "MODE-");
}

static void
keyboard_send_key (SDL_KeyboardEvent *key, void *data)
{
  struct keyboard_client_data *dcd = data;
  SDL_keysym *keysym = &(key->keysym);
  char  buffer[1024];
  char *sniplet;

  buffer[0] = '\0';
  if (key->type == SDL_KEYUP) {
    strcat (buffer, "RELEASE ");
  } else {
    strcat (buffer, "PRESS ");
  }

  cat_modifier (keysym->mod, buffer);
  asprintf (&sniplet, "\"%s\" (0x%02X)",
	    SDL_GetKeyName (keysym->sym),
	    keysym->scancode);
  strcat (buffer, sniplet);
  free (sniplet);
  
  if (key->type == SDL_KEYDOWN) {
    strcat (buffer, " -> ");
    if (keysym->unicode < 0x80 && keysym->unicode >= 32) {
      asprintf (&sniplet, "'%c' (0x%04X)",
		(char) keysym->unicode, keysym->unicode);
      strcat (buffer, sniplet);
      free (sniplet);
    } else {
      asprintf (&sniplet, "? (0x%04X)", keysym->unicode);
      strcat (buffer, sniplet);
      free (sniplet);
    }
  }
  strcat (buffer, "\n");
  buffer_puts (dcd->buf, buffer, 0, DEFAULT_COLOR);

  if (keysym->sym == SDLK_ESCAPE
      || keysym->sym == SDLK_SPACE
      || keysym->sym == SDLK_q) {
    select_vt (current_vt, 0);
  }
}

struct client_info keyboard_client = {
  init: keyboard_init,
  exit: keyboard_exit,
  activate: keyboard_activate,
  deactivate: NULL,
  send_key: keyboard_send_key,
  send_string: NULL
};
