/* messages.c - a jvterm client to keep a message log
 *
 * Copyright (C) 2003  Jochen Voss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: messages.c 6725 2005-11-09 23:15:58Z voss $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include "base.h"
#include "backend.h"
#include "ui.h"


struct message_client_data {
  struct buffer *buf;
  struct converter *conv;
  int  log_handle;

  Uint8 attr;
  Uint8 colors;
};


struct client *messages;


static const uint16_t unicode_replacement_character = 0xFFFD;

static ssize_t
emit_ucs2 (const void *data, size_t size, void *client_data)
{
  struct message_client_data *mcd = client_data;
  const uint16_t *output = data;
  int  i;

  for (i=0; i<size/sizeof(uint16_t); ++i) {
    struct charcell  cc;
    cc.c = output[i];
    cc.attr = mcd->attr;
    cc.colors = mcd->colors;
    buffer_putc (mcd->buf, cc);
  }

  return  size;
}

static void
message_write_line (enum loglevel level, const char *message,
		    void *client_data)
{
  struct message_client_data *mcd = client_data;
  
  switch (level) {
  case log_MESSAGE:
    mcd->attr = 0;
    mcd->colors = DEFAULT_COLOR;
    break;
  case log_WARNING:
    mcd->attr = 0;
    mcd->colors = JV_COLOR_PAIR (JV_COLOR_RED, JV_COLOR_BLACK);
    break;
  case log_ERROR:
    mcd->attr = JV_ATTR_BRIGHT ;
    mcd->colors = JV_COLOR_PAIR (JV_COLOR_RED, JV_COLOR_BLACK);
    break;
  }
  converter_write (mcd->conv, message, strlen(message));
  buffer_crlf (mcd->buf);
}

static void *
log_init (struct buffer *buf, void *init_data)
{
  struct message_client_data *mcd;

  mcd = xnew (struct message_client_data);
  mcd->buf = buf;
  mcd->conv = open_converter ("console12", "UTF-8", "UCS-2",
			      &unicode_replacement_character, 2,
			      emit_ucs2, mcd);
  mcd->log_handle = log_add_client (message_write_line, mcd);
  mcd->attr = 0;
  mcd->colors = DEFAULT_COLOR;
  
  return  mcd;
}

static void
log_exit (void *data)
{
  struct message_client_data *mcd = data;
  
  log_remove_client (mcd->log_handle);
  close_converter (mcd->conv);
  xfree (mcd);
}

static void
log_send_key (SDL_KeyboardEvent *key, void *data)
{
  SDL_keysym *keysym = &(key->keysym);

  if (keysym->sym == SDLK_ESCAPE
      || keysym->sym == SDLK_SPACE
      || keysym->sym == SDLK_q) {
    select_vt (current_vt, 0);
  }
}

struct client_info log_client = {
  init: log_init,
  exit: log_exit,
  activate: NULL,
  deactivate: NULL,
  send_key: log_send_key,
  send_string: NULL
};
