/* ui.h - common header file for the jvterm user interface
 *
 * Copyright (C) 2003, 2005  Jochen Voss
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: ui.h 6725 2005-11-09 23:15:58Z voss $
 */

#ifndef FILE_UI_H_SEEN
#define FILE_UI_H_SEEN

#include <SDL_events.h>
#include "backend.h"

/* from "client.c" */
struct client_info {
  void * (*init) (struct buffer *buf, void *init_data);
  void (*exit) (void *data);
  void (*activate) (void *data);
  void (*deactivate) (void *data);
  
  void (*send_key) (SDL_KeyboardEvent *key, void *data);
  ssize_t (*send_string) (Uint16 *s, size_t len, void *data);
};
struct client;

extern  struct client *new_client (const struct client_info *ci,
				   void *init_data, int backlog);
extern  void  delete_client (struct client *c);
extern  void  client_activate (struct client *c);

extern  void *client_apply (struct client *c, 
			    void *(*fn)(void *cdata, void *data),
			    void *data);

extern  void  client_send_key (SDL_KeyboardEvent *key);
extern  ssize_t  client_send_string (Uint16 *s);
extern  void  client_send_char (Uint16 c);
extern  void  client_start_select (void);

/* from "jvterm.c" */
extern  int  shutdown_flag;

extern  struct client *vt[8];
extern  int  current_vt;

struct shell_client_init {
  const char *child_charset;
  int  meta_mode;
  int  vt;
};

struct shell_client_data {
  struct console_data *con;
  pid_t  pid;
  int  fd;

  void (*fix_key) (SDL_KeyboardEvent *key);
  char *child_charset;
  struct converter *conv_input;
  int  meta_mode;
  int  vt;
};

extern  struct client_info  shell_client;

extern  void  shell_send_char (struct shell_client_data *scd, Uint16 c);
extern  void  select_vt (int n, int create);

/* from "control.c" */
extern  struct client_info  control_client;

/* from "keyboard.c" */
extern  struct client *keyclient;
extern  struct client_info  keyboard_client;

/* from "messages.c" */
extern  struct client *messages;
extern  struct client_info  log_client;

/* from "console.c" */
struct console_data;

extern  struct console_data *console_open (struct buffer *buf,
					   struct shell_client_data *scd,
					   const char *conv_id);
extern  int  console_close (struct console_data *cd);

extern  ssize_t  console_write (struct console_data *cd,
				const char *buffer, size_t size);
extern  ssize_t  console_copy (struct console_data *cd, int fd);

#endif /* FILE_UI_H_SEEN */
